@@ -27,6 +27,7 @@
 
 #if defined(NV_LINUX)
 
+#include <spawn.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdbool.h>
@@ -121,22 +122,25 @@ static int is_kernel_module_loaded(const char *nv_module_name)
 }
 
 /*
- * Attempt to redirect STDOUT and STDERR to /dev/null.
+ * Create posix_spawn rules to redirect STDOUT and STDERR to /dev/null.
  *
  * This is only for the cosmetics of silencing warnings, so do not
  * treat any errors here as fatal.
  */
-static void silence_current_process(void)
+static posix_spawn_file_actions_t* silence_process_actions(void)
 {
-    int dev_null_fd = open("/dev/null", O_RDWR);
-    if (dev_null_fd < 0)
-    {
-        return;
+    posix_spawn_file_actions_t *file_actions = malloc(sizeof *file_actions);
+
+    if (!file_actions ||
+        posix_spawn_file_actions_init(file_actions)) {
+
+        free(file_actions);
+        return NULL;
     }
 
-    dup2(dev_null_fd, STDOUT_FILENO);
-    dup2(dev_null_fd, STDERR_FILENO);
-    close(dev_null_fd);
+    posix_spawn_file_actions_addopen(file_actions, STDOUT_FILENO, "/dev/null", O_RDWR, 0);
+    posix_spawn_file_actions_adddup2(file_actions, STDOUT_FILENO, STDERR_FILENO);
+    return file_actions;
 }
 
 static bool is_tegra(void)
@@ -179,7 +183,9 @@ static int modprobe_helper(const int print_errors, const char *module_name,
     int status = 1;
     struct stat file_status;
     pid_t pid;
-    const char *envp[] = { "PATH=/sbin", NULL };
+    const char * const argv[] = { "modprobe", module_name, NULL };
+    static const char *envp[] = { "PATH=/sbin", NULL };
+    posix_spawn_file_actions_t *file_actions;
     FILE *fp;
 
     /*
@@ -209,6 +215,13 @@ static int modprobe_helper(const int print_errors, const char *module_name,
         return 1;
     }
 
+    /* Only attempt to load the kernel module if root. */
+
+    if (geteuid() != 0)
+    {
+        return 0;
+    }
+
     /*
      * Before attempting to load the module, look for any NVIDIA PCI devices.
      * If none exist, exit instead of attempting the modprobe, because doing so
@@ -238,13 +251,6 @@ static int modprobe_helper(const int print_errors, const char *module_name,
         }
     }
 
-    /* Only attempt to load the kernel module if root. */
-
-    if (geteuid() != 0)
-    {
-        return 0;
-    }
-
     /* Attempt to read the full path to the modprobe executable from /proc. */
 
     fp = fopen(NV_PROC_MODPROBE_PATH, "r");
@@ -294,55 +300,51 @@ static int modprobe_helper(const int print_errors, const char *module_name,
 
     /* Fork and exec modprobe from the child process. */
 
-    switch (pid = fork())
-    {
-        case 0:
+    file_actions = silence_process_actions();
 
-            /*
-             * modprobe might complain in expected scenarios.  E.g.,
-             * `modprobe nvidia` on a Tegra system with dGPU where no nvidia.ko is
-             * present will complain:
-             *
-             *  "modprobe: FATAL: Module nvidia not found."
-             *
-             * Silence the current process to avoid such unwanted messages.
-             */
-            silence_current_process();
+    /*
+     * POSIX specifies that the argv and envp arguments are arrays of non-const
+     * char* pointers, but that the arguments are not modified. Cast away
+     * constness here.
+     *
+     * See man 3p execv
+     */
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wcast-qual"
+    status = posix_spawn(&pid, modprobe_path, file_actions,
+                         NULL /* attrp */,
+                         (char **) argv,
+                         (char **) envp);
+#pragma GCC diagnostic pop
 
-            execle(modprobe_path, "modprobe",
-                   module_name, NULL, envp);
-
-            /* If execl(3) returned, then an error has occurred. */
-
-            if (print_errors)
-            {
-                fprintf(stderr,
-                        "NVIDIA: failed to execute `%s`: %s.\n",
-                        modprobe_path, strerror(errno));
-            }
-            exit(1);
-
-        case -1:
-            return 0;
-
-        default:
-            /*
-             * waitpid(2) is not always guaranteed to return success even if
-             * the child terminated normally.  For example, if the process
-             * explicitly configured the handling of the SIGCHLD signal
-             * to SIG_IGN, then waitpid(2) will instead block until all
-             * children terminate and return the error ECHILD, regardless
-             * of the child's exit codes.
-             *
-             * Hence, ignore waitpid(2) error codes and instead check
-             * whether the desired kernel module is loaded.
-             */
-            waitpid(pid, NULL, 0);
-
-            return is_kernel_module_loaded(module_name);
+    if (file_actions) {
+        posix_spawn_file_actions_destroy(file_actions);
+        free(file_actions);
     }
 
-    return 1;
+    if (status) {
+        if (print_errors) {
+            fprintf(stderr,
+                    "NVIDIA: failed to execute `%s`: %s.\n",
+                    modprobe_path, strerror(status));
+        }
+        return 0;
+    }
+
+    /*
+     * waitpid(2) is not always guaranteed to return success even if
+     * the child terminated normally.  For example, if the process
+     * explicitly configured the handling of the SIGCHLD signal
+     * to SIG_IGN, then waitpid(2) will instead block until all
+     * children terminate and return the error ECHILD, regardless
+     * of the child's exit codes.
+     *
+     * Hence, ignore waitpid(2) error codes and instead check
+     * whether the desired kernel module is loaded.
+     */
+    waitpid(pid, NULL, 0);
+
+    return is_kernel_module_loaded(module_name);
 }
 
 
